#!/bin/bash
######################################################################
#                                                                    #
#  Copyright (c) 2002, 2025 NoMachine, http://www.nomachine.com.     #
#                                                                    #
#  All rights reserved.                                              #
#                                                                    #
######################################################################


RunDir=${0%nxfs.sh}
RestrictedDir=$(cd "${RunDir}" && pwd)

PARAMS_COUNT=3

. "${RestrictedDir}"/nxfunct.sh


if [ -e "/sbin/kextload" ];
then
  COMMAND_KEXTLOAD="/sbin/kextload"
elif [ -e "/bin/kextload" ];
then
  COMMAND_KEXTLOAD="/bin/kextload"
else
  echo "ERROR: kextload command not found"
  exit 1
fi

if [ -e "/sbin/kextunload" ];
then
  COMMAND_KEXTUNLOAD="/sbin/kextunload"
elif [ -e "/bin/kextunload" ];
then
  COMMAND_KEXTUNLOAD="/bin/kextunload"
else
  echo "ERROR: kextunload command not found"
  exit 1
fi

if [ -e "/usr/sbin/kextstat" ];
then
  COMMAND_KEXTSTAT="/usr/sbin/kextstat"
elif [ -e "/sbin/kextstat" ];
then
  COMMAND_KEXTSTAT="/bin/kextstat"
else
  echo "ERROR: kextstat command not found"
  exit 1
fi

KEXT_PATH="/Library/Application Support/NoMachine/Extensions/nxfuse.kext"
KEXT_BUNDLE="com.nomachine.kext.nxfuse"

unloadKext()
{
  kext=`$COMMAND_KEXTSTAT -b $KEXT_BUNDLE -l`
    
  if [ "x$kext" != "x" ];
  then
    $COMMAND_KEXTUNLOAD -b "$KEXT_BUNDLE"
  else
    echo "The $KEXT_BUNDLE kext is already unloaded."
  fi

  return 0
}

loadKext()
{
  kext=`$COMMAND_KEXTSTAT -b $KEXT_BUNDLE -l`
    
  if [ "x$kext" != "x" ];
  then
    echo "The $KEXT_BUNDLE kext is already loaded."
  else
    $COMMAND_KEXTLOAD "$KEXT_PATH"
  fi

  return 0
}


case "$3" in
'--start')       loadKext   ;;
'--stop')     unloadKext ;;
esac
