#!/bin/bash
######################################################################
#                                                                    #
#  Copyright (c) 2002, 2025 NoMachine, http://www.nomachine.com.     #
#                                                                    #
#  All rights reserved.                                              #
#                                                                    #
######################################################################

RunDir=${0%nxtun.sh}
RestrictedDir=$(cd "${RunDir}" && pwd)

PARAMS_COUNT_MIN=3
PARAMS_COUNT_MAX=5

. "${RestrictedDir}"/nxfunct.sh

if test -x "/bin/echo"; then
  COMMAND_ECHO="/bin/echo"
else
  echo "NX> 500 Error: echo command not found."
  exit 1
fi

if test -x "/sbin/ifconfig"; then
  COMMAND_IFCONFIG="/sbin/ifconfig"
else
  echo "NX> 500 Error: ifconfig command not found."
  exit 1
fi

if [ -e "/sbin/kextload" ];
then
  COMMAND_KEXTLOAD="/sbin/kextload"
elif [ -e "/bin/kextload" ];
then
  COMMAND_KEXTLOAD="/bin/kextload"
else
  echo "NX> 500 Error: kextload command not found."
  exit 1
fi

if [ -e "/sbin/kextunload" ];
then
  COMMAND_KEXTUNLOAD="/sbin/kextunload"
elif [ -e "/bin/kextunload" ];
then
  COMMAND_KEXTUNLOAD="/bin/kextunload"
else
  echo "NX> 500 Error: kextunload command not found."
  exit 1
fi

if [ -e "/usr/sbin/kextstat" ];
then
  COMMAND_KEXTSTAT="/usr/sbin/kextstat"
elif [ -e "/sbin/kextstat" ];
then
  COMMAND_KEXTSTAT="/bin/kextstat"
else
  echo "NX> 500 Error: kextstat command not found."
  exit 1
fi

addInterface()
{
  ${COMMAND_IFCONFIG} $1 $2 netmask 255.255.255.252 up

  #
  # Get MAC address of created interface.
  #

  mac=$(${COMMAND_IFCONFIG})

  mac=${mac#*${1}:*ether}

  mac=$(${COMMAND_ECHO} ${mac} | ${COMMAND_CUT} -d ' ' -f 1)

  mac=${mac//":"/""}

  ${COMMAND_ECHO} "$mac"

  exit 0
}

KEXT_PATH="/Library/Application Support/NoMachine/Extensions/nxtun.kext"
KEXT_BUNDLE="com.nomachine.driver.nxtun"

unloadKext()
{
  kext=`$COMMAND_KEXTSTAT -b $KEXT_BUNDLE -l`

  if [ "x$kext" != "x" ];
  then
    $COMMAND_KEXTUNLOAD -b "$KEXT_BUNDLE"
  else
    echo "The $KEXT_BUNDLE kext is already unloaded."
  fi

  return 0
}

loadKext()
{
  kext=`$COMMAND_KEXTSTAT -b $KEXT_BUNDLE -l`

  if [ "x$kext" != "x" ];
  then
    echo "The $KEXT_BUNDLE kext is already loaded."
  else
    $COMMAND_KEXTLOAD "$KEXT_PATH"
  fi

  $COMMAND_CHMOD 666 /dev/nxtap*

  return 0
}

case "$3" in
'--add')       addInterface "$4" "$5"     ;;
'--start')       loadKext   ;;
'--stop')     unloadKext ;;
esac

