#!/bin/bash

printLogMsg ()
{
  echo "NX> 702 "$1"."
}

printErrorLogMsg ()
{
  echo "NX> 702 ERORR: "$1"."
}

printWarnLogMsg ()
{
  echo "NX> 702 WARNING: "$1"."
}

removeUser ()
{
#  if [ -d "/Library/Application Support/NoMachine/var/$1" ];
#  then
    printLogMsg "Deleting user: $1 from the system"

    if [ "x$1" = "xnx" ]; then
      dscl_out=`dscl . list /Groups | grep com.apple.access_ssh | grep -v com.apple.access_ssh-disabled 2>/dev/null`
      if [ "x${dscl_out}" != "x" ];
      then
        dscl_out=`dscl . -delete /Groups/com.apple.access_ssh GroupMembership $1 2>&1`
        if [ $? != 0 ]; then
          printErrorLogMsg "Cannot remove user $1 from the group icom.apple.access_ssh: $dscl_out"
        fi
      fi
    fi

    dscl_out=`dscl . -delete /Users/$1 2>&1`
    if [ $? != 0 ]; then
      dscl_out2=`dscl . -read /Users/$1 2>&1`
      if [ $? = 0 ]; then
        printErrorLogMsg "Cannot remove user $1: $dscl_out"
      fi
    else
      defaults write /Library/Preferences/com.apple.loginwindow HiddenUsersList -array $(defaults read /Library/Preferences/com.apple.loginwindow HiddenUsersList | grep -v -e $1 -e '(' -e ')' | tr ,'\n' ' ')
    fi

    dscl_out=`dscl . -delete /Groups/$1 2>&1`
    if [ $? != 0 ]; then
      dscl_out2=`dscl . -read /Users/$1 2>&1`
      if [ $? = 0 ]; then
        printErrorLogMsg "Cannot remove group $1: $dscl_out" 
      fi
    fi

    rm_out=`rm -fr "/Library/Application Support/NoMachine/var/$1" 2>&1`
    if [ -d "/Library/Application Support/NoMachine/var/$1" ]; then
      printErrorLogMsg "Cannot remove user $1 home folder: $rm_out" 
    else
      printLogMsg "Removed user $1 home folder: /Library/Application Support/NoMachine/var/$1" 
    fi

#  fi

}

getProcessPID ()
{
  printLogMsg "Retrieving the process PIDs: $1" 
  tab_PID=`ps ax -o "pid,command" | grep "/Applications/NoMachine.app/.*$1" | grep -v grep | awk '{print $1;}' 2>/dev/null`
}

killNXApp ()
{
  nxapp="$1"

  printLogMsg "Checking process to kill: ${nxapp}"

  if [ "x${nxapp}" = "x" ];
  then
    return
  fi

  printLogMsg "Running killall: ${nxapp}" 
  out_kill=`killall "${nxapp}" 2>&1`

  getProcessPID "${nxapp}"

  for i in $tab_PID; do
      printLogMsg "Killing process: $i" 
      kill -9 $i 2>/dev/null
  done
}

sendSigTermToNXApp ()
{
  nxapp="$1"

  printLogMsg "Checking process to send SIGTERM: ${nxapp}"

  if [ "x${nxapp}" = "x" ];
  then
    return
  fi

  getProcessPID "${nxapp}"

  for i in $tab_PID; do
    printLogMsg "Sending SIGTERM to process: $i"                  
    kill -15 $i 2>/dev/null
  done
}

setMacOSVer()
{
  macOSVerMajor=$(sw_vers -productVersion | cut -f1 -d '.')
  macOSVerMinor=$(sw_vers -productVersion | cut -f2 -d '.')
}

loadUninstallAgent()
{
  for loggedUsers in $(ps aux | grep loginwindow | grep -v  grep | awk '{print $1}');
  do
    userId=$(id -u $loggedUsers)
    launchctl bootstrap gui/$userId "/Library/LaunchAgents/com.nomachine.uninstallAgent.plist"
    launchctl kickstart -k gui/$userId/com.nomachine.uninstallAgent
  done
}

uninstallCluster()
{
  cluster_file="/Library/Application Support/NoMachine/var/run/nxcluster.interface"

  if [ -f "$cluster_file" ];
  then
    IFS=$'\n' read -d '' -r -a lines < "$cluster_file"

    length=${#lines[@]}

    for (( i=0; i < ${length}; i=i+2 ))
    do
      ifconfig "${lines[$i]}" -alias "${lines[$i+1]}"
    done
  fi
}

NM_INSTALL_PATH="/Applications/NoMachine.app"

if [ -x "/bin/launchctl" ];
then
  LAUNCHCTLCOMMAND="/bin/launchctl"
else
  LAUNCHCTLCOMMAND="launchctl"
fi

if [ ! -d "$NM_INSTALL_PATH" ];
then
  printLogMsg "Starting uninstall at: `date`"

  setMacOSVer

  serverPid=`ps ax -o "pid,command" | grep "NoMachine.*nxserver" | head -1 | awk '{print $1;}' 2>/dev/null`

  trashDir=""

  if [ "x$serverPid" != "x" ];
  then
    trashDir=`lsof -p $serverPid -F n | grep libnx.dylib | sed -e 's:^n/:/:' | sed -e 's:/Contents.*::g'`
  fi


  set -f
  IFS='
'

  for line in $(ps -ef | grep loginwindow.app | grep -v grep | awk -v cmdlaunch=$LAUNCHCTLCOMMAND '{print (cmdlaunch " asuser " $1 " " cmdlaunch " unload -F -S Aqua /Library/LaunchAgents/com.nomachine.localnxserver.plist")}');
  do
    out=$(echo $line | grep -e "asuser root" -e "asuser 0" 2>/dev/null || test 1)

    if [ "x${out}" != "x" ];
    then
      line=$(echo $line | sed 's/Aqua/LoginWindow/')

      out=$(eval $line 2>&1)

      if [ $? != 0 ];
      then
        line=$(echo $line | sed 's/LoginWindow/Aqua/')

        out=$(eval $line 2>&1)

        if [ $? != 0 ];
        then
          printErrorLogMsg "Cannot remove: com.nomachine.localnxserver.plist: $out"
        fi
      fi
    else
      out=$(eval $line 2>&1)

      if [ $? != 0 ];
      then
        printErrorLogMsg "Cannot remove: com.nomachine.localnxserver.plist: $out"
      fi
    fi
  done;

  out="$($LAUNCHCTLCOMMAND list com.nomachine.localnxserver)"

  if [ $? != 0 ];
  then

    for line in $(id -u $( ps aux | grep loginwindow.app | grep -v grep | awk '{print $1}') | awk -v cmdlaunch=$LAUNCHCTLCOMMAND '{print (cmdlaunch " asuser "$1" " cmdlaunch " unload -F -S Aqua /Library/LaunchAgents/com.nomachine.localnxserver.plist")}');
    do
      out=$(echo $line | grep "asuser root" 2>/dev/null || test 1)

      if [ "x${out}" != "x" ];
      then
	line=$(echo $line | sed 's/Aqua/LoginWindow/')

	out=$(eval $line 2>&1)

	if [ $? != 0 ];
	then
          line=$(echo $line | sed 's/LoginWindow/Aqua/')

          out=$(eval $line 2>&1)

          if [ $? != 0 ];
          then
            printErrorLogMsg "Cannot remove: com.nomachine.localnxserver.plist: $out"
          fi
	fi
      else
        out=$(eval $line 2>&1)

        if [ $? != 0 ];
        then
          printErrorLogMsg "Cannot remove: com.nomachine.localnxserver.plist: $out"
        fi
      fi
    done;
  fi

  out="$($LAUNCHCTLCOMMAND list com.nomachine.localnxserver)"

  if [ $? != 0 ];
  then
    printErrorLogMsg "Failed to remove: com.nomachine.localnxserver.plist. Using bsexec"

    for line in $(ps aux | grep loginwindow.app | grep -v grep | awk -v cmdsudo=sudo -v cmdlaunch=$LAUNCHCTLCOMMAND '{print (cmdsudo " " cmdlaunch " bsexec "$2" " cmdsudo " -u "$1" " cmdlaunch " unload -S Aqua /Library/LaunchAgents/com.nomachine.localnxserver.plist")}');
    do
      out=$(eval $line 2>&1)

      if [ $? = 0 ];
      then
        printLogMsg "Removed: com.nomachine.localnxserver.plist"
      else
        printErrorLogMsg "Cannot remove: com.nomachine.localnxserver.plist: $out"
      fi

      out=$(echo $line | grep "u root" 2>/dev/null || test 1)

      if [ "x${out}" != "x" ];
      then

        line=$(echo $line | sed 's/Aqua/LoginWindow/')

        out=$($line 2>&1)

        if [ $? = 0 ];
        then
          printLogMsg "Removed: com.nomachine.localnxserver.plist"
        else
          printErrorLogMsg "Cannot remove: com.nomachine.localnxserver.plist: $out"
        fi

      fi
    done;
  else
    printLogMsg "Removed: com.nomachine.localnxserver.plist"
  fi

  unset IFS
  set +f

  rm -f /Library/LaunchAgents/com.nomachine.localnxserver.plist

  out=`launchctl unload /Library/LaunchDaemons/com.nomachine.server.plist 2>&1`
  if [ $? = 0 ];
  then
    printLogMsg "Removed: com.nomachine.server.plist"
  else
    printErrorLogMsg "Cannot remove: com.nomachine.server.plist: $out"
  fi
  rm -f /Library/LaunchDaemons/com.nomachine.server.plist

  out=`launchctl unload -w /Library/LaunchDaemons/com.nomachine.nxd.plist 2>&1`
  if [ $? = 0 ];
  then
    printLogMsg "Removed: com.nomachine.nxd.plist"
  else
    printErrorLogMsg "Cannot remove: com.nomachine.nxd.plist: $out"
  fi
  rm -f /Library/LaunchDaemons/com.nomachine.nxd.plist
  
  out=`launchctl unload -w /Library/LaunchDaemons/com.nomachine.nxhtd.plist 2>&1`
  if [ $? = 0 ];
  then
    printLogMsg "Removed: com.nomachine.nxhtd.plist"
  else
    printErrorLogMsg "Cannot remove: com.nomachine.nxhtd.plist: $out"
  fi

  rm -f /Library/LaunchDaemons/com.nomachine.nxhtd.plist

  uninstallCluster

  sendSigTermToNXApp "nxserver.bin --startup"
  sendSigTermToNXApp "nxserver.bin --localsession"
  sendSigTermToNXApp "nxwebplayer.bin"
  sendSigTermToNXApp "nxwebclient.bin"
  sendSigTermToNXApp "nxwebrunner.bin"
  sendSigTermToNXApp "nxd"
  sendSigTermToNXApp "nxnode.bin"
  sendSigTermToNXApp "nxserver"
  sendSigTermToNXApp "nxserver.bin"
  sendSigTermToNXApp "nxhtd.bin"
  sendSigTermToNXApp "nxsh"
  sendSigTermToNXApp "nxlocate"

  killNXApp "nxserver.bin --localsession"
  killNXApp "nxwebclient.bin"
  killNXApp "nxwebrunner.bin"
  killNXApp "nxwebplayer.bin"
  killNXApp "nxd"
  killNXApp "nxnode.bin"
  killNXApp "nxserver"
  killNXApp "nxserver.bin"
  killNXApp "nxhtd.bin"
  killNXApp "nxsh"
  killNXApp "nxlocate"

  if [[ -d "$trashDir"  && "$trashDir" == *Trash/NoMachine* ]];
  then
    out=`rm -rf "$trashDir" 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Removed: NoMachine.app from trash"
    else
      printErrorLogMsg "Cannot remove NoMachine.app from trash: $out"
    fi
  fi

  AVC_UNINSTALL_DIR="/Library/Application Support/NoMachine/codec/NXAVCUninstall.app/Contents/MacOS"

  if [[ -d "${AVC_UNINSTALL_DIR}" ]];
  then
    $("${AVC_UNINSTALL_DIR}/NXAVCUninstall")
	ret=$?
	if [ $ret == 0 ];
	then
	  rm -rf "/Library/Application Support/NoMachine/codec/"
	  printLogMsg "Removed: AVC Pack."
	else
	  printLogMsg "Skipping AVC Pack removal."
	fi
  fi

  LSREGISTERCOMMAND="/System/Library/Frameworks/CoreServices.framework/Versions/A/Frameworks/LaunchServices.framework/Versions/A/Support/lsregister"

  if [ -x "$LSREGISTERCOMMAND" ];
  then

    #
    # There was nxnode.app in some point, we are leaving it to clean it after update from older packages.
    #

    "$LSREGISTERCOMMAND" -u -R "$NM_INSTALL_PATH/Contents/Frameworks/bin/nxnode.app"

    "$LSREGISTERCOMMAND" -u -R "$NM_INSTALL_PATH/Contents/Frameworks/bin/nxserver.app"

  fi

  if [ $macOSVerMajor -ge 12 ];
  then
    loadUninstallAgent
  else
    removeUser "nx"
    removeUser "nxhtd"
  fi

  rm -fr /etc/NX
  printLogMsg "Removed: /etc/NX"

  rm -f /etc/pam.d/nx
  printLogMsg "Removed PAM script: /etc/pam.d/nx"

  LCTLLIST=`launchctl list | grep com.nomachine.nxusbd`
  if [ "${LCTLLIST}x" != "x" ];
  then
    launchctl remove "com.nomachine.nxusbd"

    LCTLLIST=`launchctl list | grep com.nomachine.nxusbd`
    if [ "${LCTLLIST}x" = "x" ];
    then
      printLogMsg "Removed: com.nomachine.nxusbd.plist"
    else
      printErrorLogMsg "Cannot remove: com.nomachine.nxusbd.plist"
    fi
  fi

  LCTLLIST=`launchctl list | grep com.nomachine.nxusb.daemon`
  if [ "${LCTLLIST}x" != "x" ];
  then
    launchctl remove "com.nomachine.nxusb.daemon"

    LCTLLIST=`launchctl list | grep com.nomachine.nxusb.daemon`
    if [ "${LCTLLIST}x" = "x" ];
    then
      printLogMsg "Removed: com.nomachine.nxusb.plist"
    else
      printErrorLogMsg "Cannot remove: com.nomachine.nxusb.plist"
    fi
  fi

  KEXTST=`kextstat | grep nxusb.helper`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload "/Library/Application Support/NoMachine/Extensions/nxusb_helper.kext" 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: nxusb_helper.kext"
    else
      printErrorLogMsg "Cannot unload kext: nxusb_helper.kext: $out"
    fi
  fi

  KEXTST=`kextstat | grep nxusb.helper`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload /Library/Extensions/nxusb_helper.kext 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: nxusb_helper.kext"
    else
      printErrorLogMsg "Cannot unload kext: nxusb_helper.kext: $out"
    fi
  fi
 
  KEXTST=`kextstat | grep nxusb.io`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.driver.nxusb.io 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: com.nomachine.driver.nxusb.io"
    else
      printErrorLogMsg "Cannot unload kext: com.nomachine.driver.nxusb.io: $out"
    fi
  fi

  KEXTST=`kextstat | grep nxusb`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.driver.nxusb 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: com.nomachine.driver.nxusb"
    else
      printErrorLogMsg "Cannot unload kext: com.nomachine.driver.nxusb: $out"
    fi
  fi

  KEXTST=`kextstat | grep com.nomachine.kext.nxlogger`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.kext.nxlogger 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: com.nomachine.kext.nxlogger"
    else
      printErrorLogMsg "Cannot unload kext: com.nomachine.kext.nxlogger: $out"
    fi
  fi

  KEXTST=`kextstat | grep com.nomachine.kext.vic`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.kext.vic 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: com.nomachine.kext.vic"
    else
      printErrorLogMsg "Cannot unload kext: com.nomachine.kext.vic: $out"
    fi
  fi

  KEXTST=`kextstat | grep com.nomachine.kext.vusb`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.kext.vusb 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: com.nomachine.kext.vusb"
    else
      printErrorLogMsg "Cannot unload kext: com.nomachine.kext.vusb: $out"
    fi
  fi

  KEXTST=`kextstat | grep nxtun`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.driver.nxtun 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: nxtun.kext"
    else
      printErrorLogMsg "Cannot unload kext: nxtun.kext: $out"
    fi
  fi

  rm -rf /System/Library/StartupItems/nxtun

  KEXTST=`kextstat | grep nxfuse`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.kext.nxfuse 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: nxfuse.kext"
    else
      printErrorLogMsg "Cannot unload kext: nxfuse.kext: $out"
    fi
  fi

  rm -rf /System/Library/StartupItems/nxfuse

  KEXTST=`kextstat | grep nxau`

  if [ "${KEXTST}x" != "x" ];
  then
    out=`kextunload -b com.nomachine.driver.nxau 2>&1`
    if [ $? = 0 ];
    then
      printLogMsg "Unloaded kext: nxaudio.kext"
    else
      printErrorLogMsg "Cannot unload kext: nxaudio.kext: $out"
    fi
  fi

  if [ -d /System/Library/Extensions/nxnetwork.kext ];
  then
    KEXTST=`kextstat | grep nxnetwork`

    if [ "${KEXTST}x" != "x" ];
    then
      out=`kextunload /System/Library/Extensions/nxnetwork.kext 2>&1`
      if [ $? = 0 ];
      then
        printLogMsg "Unloaded kext: nxnetwork.kext"
      else
        printErrorLogMsg "Cannot unload kext: nxnetwork.kext: $out"
      fi
    fi
  fi

  if [ -d /System/Library/StartupItems/nxnetwork ];
  then
    rm -rf /System/Library/StartupItems/nxnetwork
  fi

  if [ -d /Library/Extensions/nxnetwork.kext ];
  then
    KEXTST=`kextstat | grep nxnetwork`

    if [ "${KEXTST}x" != "x" ];
    then
      out=`kextunload /Library/Extensions/nxnetwork.kext 2>&1`
      if [ $? = 0 ];
      then
        printLogMsg "Unloaded kext: nxnetwork.kext"
      else
        printErrorLogMsg "Cannot unload kext: nxnetwork.kext: $out"
      fi
    fi
  fi

  if [ -d /Library/StartupItems/nxnetwork ];
  then
    rm -rf /Library/StartupItems/nxnetwork
  fi

  if [ -d /System/Library/Extensions/nxdisk.kext ];
  then
    KEXTST=`kextstat | grep nxdisk`

    if [ "${KEXTST}x" != "x" ];
    then
      out=`kextunload /System/Library/Extensions/nxdisk.kext 2>&1`
      if [ $? = 0 ];
      then
        printLogMsg "Unloaded kext: nxdisk.kext"
      else
        printErrorLogMsg "Cannot unload kext: nxdisk.kext: $out"
      fi
    fi
  fi

  if [ -d /System/Library/StartupItems/nxdisk ];
  then
    rm -rf /System/Library/StartupItems/nxdisk
  fi

  if [ -d /Library/Extensions/nxdisk.kext ];
  then
    KEXTST=`kextstat | grep nxdisk`

    if [ "${KEXTST}x" != "x" ];
    then
      out=`kextunload /Library/Extensions/nxdisk.kext 2>&1`
      if [ $? = 0 ];
      then
        printLogMsg "Unloaded kext: nxdisk.kext"
      else
        printErrorLogMsg "Cannot unload kext: nxdisk.kext: $out"
      fi
    fi
  fi

  if [ -d /Library/StartupItems/nxdisk ];
  then
    rm -rf /Library/StartupItems/nxdisk
  fi

  rm -rf "/Library/Application Support/NoMachine/Extensions/nxusb_helper.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxusb_io1.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxusb_io2.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxusb.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxusb2.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxlogger.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxvhci-10_11.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxvic-10_10.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxtun.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions/nxfuse.kext"
  rm -rf "/Library/Application Support/NoMachine/Extensions"
  rm -rf /System/Library/Extensions/nxusb.kext
  rm -rf /System/Library/Extensions/nxusb_*.kext
  rm -rf /Library/Extensions/nxusb.kext
  rm -rf /Library/Extensions/nxusb_*.kext
  rm -rf /System/Library/Extensions/nxusb.kext
  rm -rf /System/Library/Extensions/nxusb_*.kext
  rm -rf /System/Library/Extensions/nxtun.kext
  rm -rf /System/Library/Extensions/nxfuse.kext
  rm -rf /System/Library/Extensions/nxaudio.kext

  rm -rf /Library/Extensions/nxtun.kext
  rm -rf /Library/Extensions/nxfuse.kext
  rm -rf /Library/Extensions/nxaudio.kext

  rm -rf /System/Library/Extensions/nxfuse.unsigned.kext
  rm -rf /System/Library/Extensions/nxaudio.unsigned.kext
  rm -rf /System/Library/Extensions/nxusb-legacy_10.7.unsigned.kext

  rm -rf /Library/Audio/Plug-Ins/HAL/NMAudio.driver
  rm -rf /Library/Audio/Plug-Ins/HAL/NMAudioMic.driver

  if [ $major_Macver -gt 14 ] || ([ $major_Macver -eq 14 ] && [ $minor_Macver -ge 4 ]);
  then
    killall coreaudiod
    ${LAUNCHCTLCOMMAND} start com.apple.audio.coreaudiod
  else
    ${LAUNCHCTLCOMMAND} stop com.apple.audio.coreaudiod
    ${LAUNCHCTLCOMMAND} start com.apple.audio.coreaudiod
  fi

  rm -rf /System/Library/Extensions/nxnetwork.kext
  rm -rf /Library/Extensions/nxnetwork.kext
  rm -rf /System/Library/Extensions/nxdisk.kext
  rm -rf /Library/Extensions/nxdisk.kext

  if [ "$macOSVerMajor" -gt "10" ] || ([ "$macOSVerMajor" -eq "10" ] && [ "$macOSVerMinor" -ge "15" ]);
  then
    kextcache -prune-staging
  fi

  CUPS_PATH=(/usr /opt/sfw/cups /usr/local /usr/lib /usr/lib64 /opt/csw /usr/libexec)

  CUPS_BACKEND=""

  for tmp in "${CUPS_PATH[@]}"; do
    if test -d "${tmp}/cups/backend" ;
    then
      CUPS_BACKEND="${tmp}/cups/backend";
      break;
    elif test -d "${tmp}/lib64/cups/backend" ;
    then
      CUPS_BACKEND="${tmp}/lib64/cups/backend";
      break;
    elif test -d "${tmp}/lib/cups/backend" ;
    then
      CUPS_BACKEND="${tmp}/lib/cups/backend";
      break;
    elif test -d "${tmp}/cups/lib/cups/backend" ;
    then
      CUPS_BACKEND="${tmp}/cups/lib/cups/backend";
      break;
    fi
  done

  if [ -d ${CUPS_BACKEND} ];
  then
    rm -f ${CUPS_BACKEND}/nxprint
    rm -f ${CUPS_BACKEND}/nx
    printLogMsg "Removed CUPS backend"
  fi

  killNXApp "nxclient"
  killNXApp "nxrunner"
  killNXApp "nxplayer"
  killNXApp "nxdock"
  killNXApp "nxhtd"
  killNXApp "nxc"
  killNXApp "nxd"
  killNXApp "nxlpd"
  killNXApp "nxlocate"
  killNXApp "nxprint"
  killNXApp "nxfsserver"
  killNXApp "nxfs"
  killNXApp "nxexec"
  killNXApp "nxkb"
  killNXApp "nxnode"
  killNXApp "nxnode.bin"
  killNXApp "nxssh-keygen"
  killNXApp "nxpost"
  killNXApp "nxauth"
  killNXApp "nxkeygen"
  killNXApp "nxsh"
  killNXApp "nxusbd"
  killNXApp "nxwebplayer"
  killNXApp "nxwebclient"
  killNXApp "nxwebrunner"
  killNXApp "nxwebplayer.bin"
  killNXApp "nxserver"

  rm -fr /Applications/NoMachine*

  rm -fr /var/db/receipts/com.nomachine*
  rm -fr /Library/Receipts/nxclient*
  rm -fr /Library/Receipts/nxrunner*
  rm -fr /Library/Receipts/nxplayer*
  rm -fr /Library/Receipts/nxnode*
  rm -fr /Library/Receipts/nxserver*
  rm -fr /Library/Receipts/nxwebplayer*
  printLogMsg "Cleaning info in packager manager database"

  #only in DEBUG version
  printLogMsg "Processes which still alive"
  ps ax -o "pid,command" | grep nx | grep -v grep | grep -v nxuninstall.sh

  printLogMsg "Uninstall completed at: `date`"

  rm -fr "/Library/Application Support/NoMachine/var"
  rm -fr "/Library/Application Support/NoMachine/nxuninstall.sh"

  rm -f /Library/LaunchDaemons/com.nomachine.uninstall.plist
  out=`launchctl remove com.nomachine.uninstall 2>&1`

  if [ "$macOSVerMajor" -ge 12 ];
  then

    osascript -e 'display dialog "NoMachine uninstallation." buttons {"Continue"} default button "Continue"'

    for loggedUsers in $(ps aux | grep loginwindow | grep -v  grep | awk '{print $1}');
    do
      launchctl bootout gui/$(id -u $loggedUsers) "/Library/LaunchAgents/com.nomachine.uninstallAgent.plist"
    done

    rm -rf "/Library/LaunchAgents/com.nomachine.uninstallAgent.plist"
  else
    rm -fr "/Library/Application Support/NoMachine"
  fi
fi

